﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;

namespace Snake__Teil_2_
{
    class Program
    {
        // Voreinstellungen (Konstanten)
        // =============================
        // Fenstergröße festlegen
        const int windowWidth = 80;
        const int windowHeight = 40;

        // Unterer Rand für Spielinfos
        const int bottomBorder = 10;

        // Hintergrundfarbe Spielfeld
        const ConsoleColor gameBackgroundColor = ConsoleColor.White;

        // Vordergrundfarbe Spielfeldrand
        const ConsoleColor gameBorderColor = ConsoleColor.Black;

        // Zeichen für Spielfeldrand
        const char gameBorderCharacter = '#';

        // Maximale Länge der Schlange
        const int maxLengthOfSnake = 250;

        // Zeichen für Schlangenkörper
        const char snakeBodyCharacter = 'S';

        // Maximale Anzahl von Futterhappen
        const int maxAmountOfSnakeFood = 30;

        // Zeichen für Futterhappen
        const char snakeFoodCharacter = 'O';

        static void Main(string[] args)
        {
            // =============================================
            // Variablen
            // =============================================
            // Zum Ziehen von Zufallszahlen für Futterhappen
            Random randomNumbersGenerator = new Random();

            // =============================================
            // Informationen zum Spiel
            // =============================================
            // Ist das Spiel noch aktiv?
            bool isGameActive = false;
            // =============================================

            // =============================================
            // Informationen zur Anzeige
            // =============================================
            // Der Inhalt dieses Arrays wird zur Darstellung
            // der aktuellen Spielsituation ausgegeben
            // gameArea[rows, columns]
            char[,] gameArea = new char[windowHeight - bottomBorder, windowWidth];
            // =============================================

            // =============================================
            // Informationen zur Schlange
            // =============================================
            // Enthält die Positionen des Schlangenkopfes 
            // und Schlangenkörpers als Zeilen/Spalten Wertepaare
            // Die Werte müssen größer 0 sein
            int[,] positionsOfSnake = new int[maxLengthOfSnake, 2];

            // Aktuelle Gesamtlänge der Schlange
            int lengthOfSnake = 4;
            // =============================================

            // =============================================
            // Informationen zu den Futterhappen
            // =============================================
            // Enthält die Position und Wert aller Futterhappen
            // Maximal 100 Futterhappen sind möglich
            // positionsOfSnakeFood[row, column, value]
            int[,] positionsOfSnakeFood = new int[maxAmountOfSnakeFood, 3];

            // Anzahl der Futterhappen
            int amountOfSnakeFood = 0;
            // =============================================

            // =============================================
            // Initialisierung des Spiels
            // Fenster initialisieren
            initializeWindow();

            // Spielfeld initialisiere
            initializeGameArea(gameArea);

            // Startpositionen der Schlange
            initializeSnakePosition(positionsOfSnake, lengthOfSnake);

            // Schlange in Array eintragen
            drawSnake(positionsOfSnake, lengthOfSnake, gameArea);
            // =============================================

            // =============================================
            // Beginn der Spiellogik
            // =============================================
            // Spiel starten
            isGameActive = true;
            // Inhalt des Spielfeldes und Statusinformationen
            // auf Bildschirm ausgeben
            drawScreen(gameArea);

            // Ablauf des Spieles
            do
            {
                // Wartezeit
                Thread.Sleep(100);

                // Futterhappen erzeugen
                amountOfSnakeFood = createSnakeFood(randomNumbersGenerator,
                                                  positionsOfSnake,
                                                  lengthOfSnake,
                                                  positionsOfSnakeFood,
                                                  amountOfSnakeFood);

                // Eintragen der Position der Futterhappen im Spielfeld
                drawSnakeFood(positionsOfSnakeFood, amountOfSnakeFood, gameArea);

                // Inhalt des Spielfeldes auf dem Bildschirm ausgeben
                drawScreen(gameArea);
            }
            while (isGameActive == true);

        }

        // Methoden zu Initialisierung
        // =======================================================
        // Konsolenfenster initialisieren
        static void initializeWindow()
        {
            // Fenster einstellen
            Console.SetWindowSize(1, 1);
            Console.SetBufferSize(windowWidth, windowHeight);
            Console.SetWindowSize(windowWidth, windowHeight);
            Console.Title = "Snake (by Uwe Homm)";

            Console.BackgroundColor = gameBackgroundColor;
            Console.CursorVisible = false;
            Console.Clear();
        }

        // Spielfeld initialisieren
        static void initializeGameArea(char[,] gameArea)
        {
            // Array mit Leerzeichen vorbelegen
            for (int row = 0; row < windowHeight - bottomBorder; row++)
            {
                for (int column = 0; column < windowWidth; column++)
                {
                    gameArea[row, column] = ' ';
                }
            }
            // Zeichnen der Spielfeldumrandung
            // Oberen und unteren Rand zeichnen
            for (int xPos = 0; xPos < windowWidth; xPos++)
            {
                gameArea[0, xPos] = gameBorderCharacter;
                gameArea[windowHeight - bottomBorder - 1, xPos] = gameBorderCharacter;
            }

            // Linken und rechten Rand zeichnen
            for (int yPos = 1; yPos < windowHeight - bottomBorder - 1; yPos++)
            {
                Console.SetCursorPosition(0, yPos);
                gameArea[yPos, 0] = gameBorderCharacter;
                gameArea[yPos, windowWidth - 1] = gameBorderCharacter;
            }
        }

        // Anfangsposition der Schlange initialisieren
        static void initializeSnakePosition(int[,] positionsOfSnake,
                                            int lengthOfSnake)
        {
            // Kopf in der Mitte
            int row = (windowHeight - bottomBorder) / 2;
            int column = windowWidth / 2;

            // Position Körper speichern
            for (int i = 0; i < lengthOfSnake; i++)
            {
                positionsOfSnake[i, 0] = row;
                positionsOfSnake[i, 1] = column + i;
            }
        }

        // Methoden zur Spiellogik
        // =======================================================
        // Gibt den Inhalt des Spielfeldes im Fenster aus
        static void drawScreen(char[,] gameArea)
        {
            // Einstellen der Farben im Konsolenfenster
            Console.BackgroundColor = gameBackgroundColor;
            Console.ForegroundColor = gameBorderColor;

            // Hier wird der gesamte Inhalt des Array gameArea
            // tatsächlich in das Fenster geschrieben
            for (int row = 0; row < windowHeight - bottomBorder; row++)
            {
                for (int column = 0; column < windowWidth; column++)
                {
                    Console.SetCursorPosition(column, row);
                    Console.Write(gameArea[row, column]);
                }
            }
        }

        // Überträgt die Position der Schlange in das Spielfeld
        static void drawSnake(int[,] positionsOfSnake, int lengthOfSnake,
                                      char[,] gameArea)
        {
            // Den Schlangenkörper in die Spielsituation eintragen
            for (int i = 0; i < lengthOfSnake; i++)
            {
                // Zeile und Spalte ermitteln
                int row = positionsOfSnake[i, 0];
                int column = positionsOfSnake[i, 1];
                // Zeichen für Schlange an der Position eintragen
                gameArea[row, column] = snakeBodyCharacter;

                // Kompakter, aber schwerer lesbar!
                // gameArea[positionsOfSnake[i, 0], positionsOfSnake[i, 1]] = snakeBodyCharacter;
            }
        }

        // Erzeugt einen neuen Futterhappen mit zufälliger Position 
        // im Spielfeld und gibt die aktuelle Anzahl der Futterhappen
        // zurück
        static int createSnakeFood(Random randomNumbersGenerator, int[,] positionsOfSnake,
                                   int lengthOfSnake, int[,] positionsOfSnakeFood,
                                   int amountOfSnakeFood)
        {
            // Keine neuen Futterhappen, wenn bereits Maximum erreicht
            if (amountOfSnakeFood >= maxAmountOfSnakeFood)
                return amountOfSnakeFood;

            // Drei Zufallszahlen für die Position
            // und den Wert des Futterhappens ziehen
            // Die Position soll innerhalb des
            // Spielfelds mit einem Abstand von 2
            // zum Rand sein
            // Somit liegt der Wert für x zwischen
            // 3 und 76 und für y zwischen 3 und 26
            // für die Größe 80x40

            int column = randomNumbersGenerator.Next(3, windowWidth - 3);
            int row = randomNumbersGenerator.Next(3, windowHeight -
                                                     bottomBorder - 3);
            int value = randomNumbersGenerator.Next(1, 11);

            // Ein Futterhappen darf nur auf
            // einem leeren Feld erzeugt werden!

            // Zuerst prüfen, ob die Position bereits
            // duch einen anderen Futterhappen belegt ist
            // Annahme: Position ist frei
            bool isPositionEmpty = true;
            // Alle Futterhappen durchlaufen
            for (int i = 0; i < amountOfSnakeFood; i++)
            {

                // Hat der Futterhappen die gleiche Position?
                if (positionsOfSnakeFood[i, 0] == row &&
                    positionsOfSnakeFood[i, 1] == column)
                {
                    // Position ist belegt
                    isPositionEmpty = false;
                    // Schleife vorzeitig beenden
                    break;
                }
            }

            // Falls kein Futterhappenposition
            if (isPositionEmpty)
            {
                // Prüfen, ob die Position bereits
                // duch den Schlangenkörper belegt ist
                for (int i = 0; i < lengthOfSnake; i++)
                {

                    if (positionsOfSnake[i, 0] == row &&
                        positionsOfSnake[i, 1] == column)
                    {
                        isPositionEmpty = false;
                        break;
                    }
                }
            }

            // Falls nicht, Futterhappen
            // am Ende hinzufügen und Anzahl inkrementieren
            if (isPositionEmpty == true)
            {
                positionsOfSnakeFood[amountOfSnakeFood, 0] = row;
                positionsOfSnakeFood[amountOfSnakeFood, 1] = column;
                positionsOfSnakeFood[amountOfSnakeFood, 2] = value;
                amountOfSnakeFood++;
            }

            return amountOfSnakeFood;
        }

        // Überträgt die Position der Futterhappen in das Spielfeld
        static void drawSnakeFood(int[,] positionsOfSnakeFood, int amountOfSnakeFood,
                                           char[,] gameArea)
        {
            for (int i = 0; i < amountOfSnakeFood; i++)
            {
                // Zeile der Futterhappenposition
                int row = positionsOfSnakeFood[i, 0];
                // Spalte der Futterhappenposition
                int column = positionsOfSnakeFood[i, 1];
                // Eintragen des Zeichen für Futterhappen
                gameArea[row, column] = snakeFoodCharacter;
            }
        }

    }
}
